@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/help.c,v 4.16 2002/01/05 02:48:01 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/help.c,v 1.2 2005/01/15 00:23:23 william Exp $
  */
 
 #define _INCLUDE_FROM_HELP_C_
@@ -62,7 +62,7 @@ void About()
    sprintf(c_ptr, TgLoadString(STID_HYPER_GRAPHICS_INFO), hyperGraphicsURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    sprintf(c_ptr, TgLoadString(STID_MAILING_LIST_INFO),
-         mailingList, joinMailingList, oldMailingList);
+         mailingList, joinMailingList, oldMailingList, mailingURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    sprintf(c_ptr, TgLoadString(STID_SEND_BUG_REPORT_TO), "bill.cheng@acm.org");
    c_ptr = (&c_ptr[strlen(c_ptr)]);
