@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/move.c,v 4.12 2001/08/06 21:39:07 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/move.c,v 1.2 2005/01/15 00:23:23 william Exp $
  */
 
 #define _INCLUDE_FROM_MOVE_C_
@@ -498,18 +498,14 @@ int ConstrainedMoveAllSel(Dx, Dy, ltx, lty, rbx, rby)
 void MoveAllSel(Dx, Dy)
    register int Dx, Dy;
 {
-   int ltx, lty, rbx, rby;
-   struct MoveSubCmdRec *move_cmd;
-   struct SubCmdRec *sub_cmd;
+   int ltx=0, lty=0, rbx=0, rby=0;
+   struct SubCmdRec *sub_cmd=NULL;
 
-   move_cmd = (struct MoveSubCmdRec *)malloc(sizeof(struct MoveSubCmdRec));
    sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
-   if (move_cmd == NULL || sub_cmd == NULL) FailAllocMessage();
-   memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
+   if (sub_cmd == NULL) FailAllocMessage();
    memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-   sub_cmd->detail.mv = move_cmd;
-   move_cmd->dx = Dx;
-   move_cmd->dy = Dy;
+   sub_cmd->detail.move.dx = Dx;
+   sub_cmd->detail.move.dy = Dy;
 
    if (moveMode==CONST_MOVE && !justDupped) {
       MarkObjectsForMove();
@@ -542,7 +538,6 @@ void MoveAllSel(Dx, Dy)
             selLtX-GRID_ABS_SIZE(1)+Dx, selLtY-GRID_ABS_SIZE(1)+Dy,
             selRbX+GRID_ABS_SIZE(1)+Dx, selRbY+GRID_ABS_SIZE(1)+Dy);
    }
-   free(move_cmd);
    free(sub_cmd);
 }
 
@@ -562,8 +557,7 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
       struct SelRec *saved_top_sel=topSel, *saved_bot_sel=botSel;
       int ltx=ObjPtr->bbox.ltx, lty=ObjPtr->bbox.lty;
       int rbx=ObjPtr->bbox.rbx, rby=ObjPtr->bbox.rby;
-      struct MoveSubCmdRec *move_cmd;
-      struct SubCmdRec *sub_cmd;
+      struct SubCmdRec *sub_cmd=NULL;
 
       topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
       if (topSel == NULL) FailAllocMessage();
@@ -572,15 +566,11 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
       topSel->obj = ObjPtr;
       UpdSelBBox();
 
-      move_cmd = (struct MoveSubCmdRec *)malloc(
-            sizeof(struct MoveSubCmdRec));
       sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
-      if (move_cmd == NULL || sub_cmd == NULL) FailAllocMessage();
-      memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
+      if (sub_cmd == NULL) FailAllocMessage();
       memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-      sub_cmd->detail.mv = move_cmd;
-      move_cmd->dx = Dx;
-      move_cmd->dy = Dy;
+      sub_cmd->detail.move.dx = Dx;
+      sub_cmd->detail.move.dy = Dy;
 
       MoveAllSelObjects(Dx, Dy);
       PrepareToRecord(CMD_MOVE, topSel, botSel, numObjSelected);
@@ -590,7 +580,6 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
             ltx-GRID_ABS_SIZE(1)+Dx, lty-GRID_ABS_SIZE(1)+Dy,
             rbx+GRID_ABS_SIZE(1)+Dx, rby+GRID_ABS_SIZE(1)+Dy);
 
-      free(move_cmd);
       free(sub_cmd);
 
       free(topSel);
@@ -1182,7 +1171,7 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
    int sel_ltx=0, sel_lty=0, sel_rbx=0, sel_rby=0;
    int ruler_ltx=0, ruler_lty=0, ruler_rbx=0, ruler_rby=0;
    int moving=TRUE, dx=0, dy=0, grid_x=OrigX, grid_y=OrigY;
-   int can_move_sub_obj=FALSE, move_sub_obj=FALSE;
+   int can_move_sub_obj=FALSE, move_sub_obj=FALSE, no_no_lock_sel=FALSE;
    char buf[80], x_buf[80], y_buf[80];
    XEvent ev;
    Time down_click_time=(Time)0;
@@ -1192,8 +1181,7 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
       down_click_time = down_button_ev->time;
    }
    if (numObjSelected == numObjLocked || ObjPtr->locked) {
-      MsgBox(TgLoadString(STID_LOCKED_OBJS_CANT_BE_MOVED), TOOL_NAME, INFO_MB);
-      return;
+      no_no_lock_sel = TRUE;
    }
    XFlush(mainDisplay);
    XSync(mainDisplay, False);
@@ -1206,16 +1194,23 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
       can_move_sub_obj = TRUE;
       SetStringStatus(TgLoadString(STID_HINT_CTL_MOVE_ATTR_ONLY));
    }
-   sel_ltx = OFFSET_X(selNoLockLtX) - 1; sel_lty = OFFSET_Y(selNoLockLtY) - 1;
-   sel_rbx = OFFSET_X(selNoLockRbX) + 1; sel_rby = OFFSET_Y(selNoLockRbY) + 1;
-
+   if (!move_sub_obj && !no_no_lock_sel) {
+      sel_ltx = OFFSET_X(selNoLockLtX)-1; sel_lty = OFFSET_Y(selNoLockLtY)-1;
+      sel_rbx = OFFSET_X(selNoLockRbX)+1; sel_rby = OFFSET_Y(selNoLockRbY)+1;
+   }
    ruler_ltx = OFFSET_X(selNoLockObjLtX); ruler_lty = OFFSET_Y(selNoLockObjLtY);
    ruler_rbx = OFFSET_X(selNoLockObjRbX); ruler_rby = OFFSET_Y(selNoLockObjRbY);
 
-   SelBox(drawWindow, revDefaultGC, sel_ltx, sel_lty, sel_rbx, sel_rby);
+   if (!move_sub_obj && !no_no_lock_sel) {
+      SelBox(drawWindow, revDefaultGC, sel_ltx, sel_lty, sel_rbx, sel_rby);
+   }
    PixelToMeasurementUnit(x_buf, 0);
    PixelToMeasurementUnit(y_buf, 0);
-   sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+   if (VerboseMeasureTooltip()) {
+      sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+   } else {
+      sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+   }
    StartShowMeasureCursor(OrigX, OrigY, buf, TRUE);
    BeginIntervalRulers(ruler_ltx, ruler_lty, ruler_rbx, ruler_rby);
 
@@ -1247,9 +1242,13 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
          EndIntervalRulers(grid_x, grid_y);
          PixelToMeasurementUnit(x_buf, ABS_SIZE(dx));
          PixelToMeasurementUnit(y_buf, ABS_SIZE(dy));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
          EndShowMeasureCursor(grid_x, grid_y, buf, TRUE);
-         if (!move_sub_obj) {
+         if (!move_sub_obj && !no_no_lock_sel) {
             SelBox(drawWindow, revDefaultGC, sel_ltx+dx, sel_lty+dy, sel_rbx+dx,
                   sel_rby+dy);
          }
@@ -1271,7 +1270,11 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
 
          PixelToMeasurementUnit(x_buf, ABS_SIZE(dx));
          PixelToMeasurementUnit(y_buf, ABS_SIZE(dy));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
          ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
          if (input.type == KeyPress || input.type == KeyRelease) {
             x = grid_x;
@@ -1295,22 +1298,49 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
 
          /* erase */
          DrawInnerRubberObj(ObjPtr, &rubber_info, dx, dy);
-         if (!saved_move_sub_obj) {
+         if (!saved_move_sub_obj && !no_no_lock_sel) {
             SelBox(drawWindow, revDefaultGC, sel_ltx+dx, sel_lty+dy, sel_rbx+dx,
                   sel_rby+dy);
          }
          dx = grid_x - OrigX;
          dy = grid_y - OrigY;
 
-         DrawIntervalRulers(ruler_ltx+dx, ruler_lty+dy, ruler_rbx+dx,
-               ruler_rby+dy);
-         if (!move_sub_obj) {
-            SelBox(drawWindow, revDefaultGC, sel_ltx+dx, sel_lty+dy, sel_rbx+dx,
-                  sel_rby+dy);
+         if ((dx != 0 || dy != 0) &&
+               (numObjSelected == numObjLocked || ObjPtr->locked)) {
+            XUngrabPointer(mainDisplay, CurrentTime);
+            XSync(mainDisplay, False);
+
+            EndIntervalRulers(grid_x, grid_y);
+            PixelToMeasurementUnit(x_buf, ABS_SIZE(dx));
+            PixelToMeasurementUnit(y_buf, ABS_SIZE(dy));
+            if (VerboseMeasureTooltip()) {
+               sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+            } else {
+               sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+            }
+            ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+            EndShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+            if (!move_sub_obj && !no_no_lock_sel) {
+               SelBox(drawWindow, revDefaultGC, sel_ltx+dx, sel_lty+dy,
+                     sel_rbx+dx, sel_rby+dy);
+            }
+            MsgBox(TgLoadString(STID_LOCKED_OBJS_CANT_BE_MOVED), TOOL_NAME,
+                  INFO_MB);
+            return;
          }
          PixelToMeasurementUnit(x_buf, ABS_SIZE(dx));
          PixelToMeasurementUnit(y_buf, ABS_SIZE(dy));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
+         DrawIntervalRulers(ruler_ltx+dx, ruler_lty+dy, ruler_rbx+dx,
+               ruler_rby+dy, buf);
+         if (!move_sub_obj && !no_no_lock_sel) {
+            SelBox(drawWindow, revDefaultGC, sel_ltx+dx, sel_lty+dy, sel_rbx+dx,
+                  sel_rby+dy);
+         }
          ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
 
          UpdateInnerRubberObj(ObjPtr, &rubber_info, dx, dy);
@@ -1322,6 +1352,11 @@ void MoveSel(OrigX, OrigY, ObjPtr, down_button_ev)
    FreeRubberInfo(ObjPtr, &rubber_info);
 
    if (dx != 0 || dy != 0) {
+      if (numObjSelected == numObjLocked || ObjPtr->locked) {
+         MsgBox(TgLoadString(STID_LOCKED_OBJS_CANT_BE_MOVED), TOOL_NAME,
+               INFO_MB);
+         return;
+      }
       HighLightReverse();
       dx = ABS_SIZE(dx);
       dy = ABS_SIZE(dy);
@@ -1769,18 +1804,55 @@ void MarkVs(Dir, Dx, Dy)
    }
 }
 
+static
+void GetSelectedVsBBox(pBBox)
+   struct BBRec *pBBox;
+{
+   int found=FALSE, ltx=0, lty=0, rbx=0, rby=0;
+   struct VSelRec *vsel_ptr=NULL;
+
+   vsel_ptr = botVSel;
+   while (vsel_ptr != NULL) {
+      int i=0;
+
+      for (i=0; i < vsel_ptr->n; i++) {
+         if (!(vsel_ptr->obj->type==OBJ_POLYGON &&
+               vsel_ptr->obj->detail.g->n-1==vsel_ptr->v_index[i])) {
+            int x=OFFSET_X(vsel_ptr->x[i]);
+            int y=OFFSET_Y(vsel_ptr->y[i]);
+
+            if (found) {
+               if (x < ltx) ltx = x;
+               if (x > rbx) rbx = x;
+               if (y < lty) lty = y;
+               if (y > rby) rby = y;
+            } else {
+               found = TRUE;
+               ltx = rbx = x;
+               lty = rby = y;
+            }
+         }
+      }
+      vsel_ptr = vsel_ptr->prev;
+   }
+   if (found) {
+      pBBox->ltx = ltx; pBBox->lty = lty; pBBox->rbx = rbx; pBBox->rby = rby;
+   }
+}
+
 void MoveSelVs(OrigX, OrigY)
    int OrigX, OrigY;
 {
-   int x, y, i;
-   struct ObjRec *obj_ptr;
-   struct VSelRec *vsel_ptr;
-   struct PolyRec *poly_ptr;
-   struct PolygonRec *polygon_ptr;
-   IntPoint *pv;
-   int moving = TRUE, dx, dy, num_pts, curved;
-   int grid_x=OrigX, grid_y=OrigY;
+   int x=0, y=0, i=0;
+   struct ObjRec *obj_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL;
+   struct PolyRec *poly_ptr=NULL;
+   struct PolygonRec *polygon_ptr=NULL;
+   IntPoint *pv=NULL;
+   int moving=TRUE, dx=0, dy=0, num_pts=0, curved=FALSE;
+   int grid_x=OrigX, grid_y=OrigY, abs_dx_from_orig=0, abs_dy_from_orig=0;
    int saved_grid_x=OrigX, saved_grid_y=OrigY;
+   struct BBRec ruler_bbox;
    char buf[80], x_buf[80], y_buf[80];
    XEvent ev;
 
@@ -1797,17 +1869,27 @@ void MoveSelVs(OrigX, OrigY)
    HighLightVs(FORWARD);
    MarkVs(FORWARD, 0, 0);
 
+   memset(&ruler_bbox, 0, sizeof(struct BBRec));
+   GetSelectedVsBBox(&ruler_bbox);
+
    PixelToMeasurementUnit(x_buf, 0);
    PixelToMeasurementUnit(y_buf, 0);
    sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+   if (VerboseMeasureTooltip()) {
+      sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+   } else {
+      sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+   }
    StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+   BeginIntervalRulers(ruler_bbox.ltx, ruler_bbox.lty, ruler_bbox.rbx,
+         ruler_bbox.rby);
 
    if (!debugNoPointerGrab) {
       XGrabPointer(mainDisplay, drawWindow, FALSE,
             PointerMotionMask | ButtonReleaseMask,
             GrabModeAsync, GrabModeAsync, None, moveCursor, CurrentTime);
    }
-   while(moving) {
+   while (moving) {
       XEvent input;
 
       XNextEvent(mainDisplay, &input);
@@ -1817,9 +1899,15 @@ void MoveSelVs(OrigX, OrigY)
       } else if (input.type == ButtonRelease) {
          XUngrabPointer(mainDisplay, CurrentTime);
          XSync(mainDisplay, False);
+
+         EndIntervalRulers(grid_x, grid_y);
          PixelToMeasurementUnit(x_buf, ABS_SIZE(grid_x-OrigX));
          PixelToMeasurementUnit(y_buf, ABS_SIZE(grid_y-OrigY));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
          EndShowMeasureCursor(grid_x, grid_y, buf, TRUE);
          MarkRulers(grid_x, grid_y);
          moving = FALSE;
@@ -1842,7 +1930,11 @@ void MoveSelVs(OrigX, OrigY)
             input.type == KeyRelease) {
          PixelToMeasurementUnit(x_buf, ABS_SIZE(grid_x-OrigX));
          PixelToMeasurementUnit(y_buf, ABS_SIZE(grid_y-OrigY));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
          ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
          MarkVs(REVERSE, grid_x-OrigX, grid_y-OrigY);
          HighLightVs(REVERSE);
@@ -1871,8 +1963,7 @@ void MoveSelVs(OrigX, OrigY)
          saved_grid_x = grid_x;
          saved_grid_y = grid_y;
 
-         MarkRulers(grid_x, grid_y);
-         for (vsel_ptr=botVSel, obj_ptr = tmpBotObj; vsel_ptr != NULL;
+         for (vsel_ptr=botVSel, obj_ptr=tmpBotObj; vsel_ptr != NULL;
                vsel_ptr=vsel_ptr->prev, obj_ptr=obj_ptr->prev) {
             switch (obj_ptr->type) {
             case OBJ_POLY:
@@ -1894,7 +1985,7 @@ void MoveSelVs(OrigX, OrigY)
                            drawOrigX, drawOrigY, num_pts, pv);
                      break;
                   case LT_INTSPLINE:
-                     free(poly_ptr->intvlist);
+                     if (poly_ptr->intvlist != NULL) free(poly_ptr->intvlist);
                      poly_ptr->svlist = MakeIntSplinePolyVertex(
                            &(poly_ptr->sn), &(poly_ptr->intn),
                            &(poly_ptr->intvlist), drawOrigX, drawOrigY,
@@ -1928,7 +2019,9 @@ void MoveSelVs(OrigX, OrigY)
                            drawOrigX, drawOrigY, num_pts, pv);
                      break;
                   case LT_INTSPLINE:
-                     free(polygon_ptr->intvlist);
+                     if (polygon_ptr->intvlist != NULL) {
+                        free(polygon_ptr->intvlist);
+                     }
                      polygon_ptr->svlist =
                            MakeIntSplinePolygonVertex(&(polygon_ptr->sn),
                            &(polygon_ptr->intn), &(polygon_ptr->intvlist),
@@ -1945,10 +2038,19 @@ void MoveSelVs(OrigX, OrigY)
             }
          }
          HighLightVs(FORWARD);
+         abs_dx_from_orig = ABS_SIZE(grid_x-OrigX);
+         abs_dy_from_orig = ABS_SIZE(grid_y-OrigY);
          MarkVs(FORWARD, grid_x-OrigX, grid_y-OrigY);
-         PixelToMeasurementUnit(x_buf, ABS_SIZE(grid_x-OrigX));
-         PixelToMeasurementUnit(y_buf, ABS_SIZE(grid_y-OrigY));
-         sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         PixelToMeasurementUnit(x_buf, abs_dx_from_orig);
+         PixelToMeasurementUnit(y_buf, abs_dy_from_orig);
+         if (VerboseMeasureTooltip()) {
+            sprintf(buf, "dx=%s dy=%s", x_buf, y_buf);
+         } else {
+            sprintf(buf, "dx=%s\ndy=%s", x_buf, y_buf);
+         }
+         DrawIntervalRulers(ruler_bbox.ltx+abs_dx_from_orig,
+               ruler_bbox.lty+abs_dy_from_orig, ruler_bbox.rbx+abs_dx_from_orig,
+               ruler_bbox.rby+abs_dy_from_orig, buf);
          ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
          while (XCheckMaskEvent(mainDisplay, PointerMotionMask, &ev)) ;
       }
