@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/remote.c,v 4.41 2002/02/22 22:55:16 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/remote.c,v 1.2 2005/01/15 00:23:23 william Exp $
  */
 
 #define _INCLUDE_FROM_REMOTE_C_
@@ -552,7 +552,7 @@ char *WriteRemoteFileIntoTemp(psz_buf, buf_sz, psz_ext)
    }
    if (psz_ext != NULL) strcat(tmp_fname, psz_ext);
    unlink(tmp_fname);
-   if ((fd=open(tmp_fname, O_WRONLY|O_CREAT|O_TRUNC)) == (-1)) {
+   if ((fd=open(tmp_fname, O_WRONLY|O_CREAT|O_TRUNC, tmpFileMode)) == (-1)) {
       sprintf(gszMsgBox, TgLoadString(STID_CANT_CREATE_NAMED_TMP_FILE),
             tmp_fname);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -1097,6 +1097,7 @@ int LoadAuthenticatedFileInMem(url, host, port, ppsz_buf, ppsz_content_type,
       sprintf(gszMsgBox, TgLoadString(STID_ENTER_USERNAME_FOR_REALM), realm);
       if (mainDisplay == NULL) {
          fprintf(stdout, "%s ", gszMsgBox);
+         fflush(stdout);
          psz = UtilGetALine(stdin);
          if (psz != NULL) {
             UtilStrCpyN(user_name, sizeof(user_name), psz);
@@ -1113,6 +1114,7 @@ int LoadAuthenticatedFileInMem(url, host, port, ppsz_buf, ppsz_content_type,
          doPassword = TRUE;
          if (mainDisplay == NULL) {
             fprintf(stdout, "%s ", gszMsgBox);
+            fflush(stdout);
             psz = UtilGetALine(stdin);
             if (psz != NULL) {
                UtilStrCpyN(password, sizeof(password), psz);
